#!/bin/bash

shopt -s extglob

prime=$1

if [[ -z $prime ]]
then
    echo "usage: inv-check.bash <prime>"
    exit 1
fi

if [[ $prime != @(17|31|47|79|97|113) ]]
then
    echo "error: prime must be one of: 17 31 47 79 97 113"
    exit 1
fi

{

    awk '/\(base '$prime'\)/ {print $1, $2, $3}' < poly-roots.txt

    echo "prime = $prime"

    echo '

define display(x) {

    a9 = x % (prime^9)
    a10 = x % (prime^10)

    if (a9 == a10) {
	print x % (prime^1), ","
	print x % (prime^2), ","
	print x % (prime^3), ","
	print x % (prime^4), ","
	print x % (prime^5), ","
	print x % (prime^6), ","
	print x % (prime^7), ","
	print x % (prime^8), ","
	print x % (prime^9), ","
	print x % (prime^10), "\n"
    }

    b9 = (prime^9) - (x % prime^9)
    b10 = (prime^10) - (x % prime^10)

    if (b9 == b10) {
	print "-", (prime^1) - (x % prime^1), ","
	print "-", (prime^2) - (x % prime^2), ","
	print "-", (prime^3) - (x % prime^3), ","
	print "-", (prime^4) - (x % prime^4), ","
	print "-", (prime^5) - (x % prime^5), ","
	print "-", (prime^6) - (x % prime^6), ","
	print "-", (prime^7) - (x % prime^7), ","
	print "-", (prime^8) - (x % prime^8), ","
	print "-", (prime^9) - (x % prime^9), ","
	print "-", (prime^10) - (x % prime^10), "\n"
    }
}

print "**************** Root evaluation ****************\n"

print "Evaluating root r1 at f(x) = x^4 - 8*x^2 + 8\n"
ignore = display(r1^4 -8*r1^2 + 8)

print "Evaluating root r2 at f(x) = x^4 - 8*x^2 + 8\n"
ignore = display(r2^4 -8*r2^2 + 8)

print "Evaluating root r3 at f(x) = x^4 - 8*x^2 + 8\n"
ignore = display(r3^4 -8*r3^2 + 8)

print "Evaluating root r4 at f(x) = x^4 - 8*x^2 + 8\n"
ignore = display(r4^4 -8*r4^2 + 8)

define sym_poly_4(e1) {
    s = 0

    s += r1^e1 * r2^e1 * r3^e1 * r4^e1

    return ( s )
}

define sym_poly_3_1(e3,e1) {
    s = 0

    s += r1^e1 * r2^e3 * r3^e3 * r4^e3
    s += r1^e3 * r2^e1 * r3^e3 * r4^e3
    s += r1^e3 * r2^e3 * r3^e1 * r4^e3
    s += r1^e3 * r2^e3 * r3^e3 * r4^e1

    return ( s )
}

define sym_poly_2_2(e2a,e2b) {
    s = 0

    s += r1^e2a * r2^e2a * r3^e2b * r4^e2b
    s += r1^e2a * r2^e2b * r3^e2a * r4^e2b
    s += r1^e2a * r2^e2b * r3^e2b * r4^e2a
    s += r1^e2b * r2^e2a * r3^e2a * r4^e2b
    s += r1^e2b * r2^e2a * r3^e2b * r4^e2a
    s += r1^e2b * r2^e2b * r3^e2a * r4^e2a

    return ( s )
}

define sym_poly_2_1_1(e2,e1a,e1b) {
    s = 0

    s += r1^e1a * r2^e1b * r3^e2  * r4^e2
    s += r1^e1a * r2^e2  * r3^e1b * r4^e2
    s += r1^e1a * r2^e2  * r3^e2  * r4^e1b
    s += r1^e1b * r2^e1a * r3^e2  * r4^e2
    s += r1^e2  * r2^e1a * r3^e1b * r4^e2
    s += r1^e2  * r2^e1a * r3^e2  * r4^e1b
    s += r1^e1b * r2^e2  * r3^e1a * r4^e2
    s += r1^e2  * r2^e1b * r3^e1a * r4^e2
    s += r1^e2  * r2^e2  * r3^e1a * r4^e1b
    s += r1^e1b * r2^e2  * r3^e2  * r4^e1a
    s += r1^e2  * r2^e1b * r3^e2  * r4^e1a
    s += r1^e2  * r2^e2  * r3^e1b * r4^e1a

    return ( s )
}

define sym_poly_1_1_1_1(e1a,e1b,e1c,e1d) {
    s = 0

    s += r1^e1a * r2^e1b * r3^e1c * r4^e1d
    s += r1^e1a * r2^e1b * r3^e1d * r4^e1c
    s += r1^e1a * r2^e1c * r3^e1b * r4^e1d
    s += r1^e1a * r2^e1c * r3^e1d * r4^e1b
    s += r1^e1a * r2^e1d * r3^e1b * r4^e1c
    s += r1^e1a * r2^e1d * r3^e1c * r4^e1b

    s += r1^e1b * r2^e1a * r3^e1c * r4^e1d
    s += r1^e1b * r2^e1a * r3^e1d * r4^e1c
    s += r1^e1b * r2^e1c * r3^e1a * r4^e1d
    s += r1^e1b * r2^e1c * r3^e1d * r4^e1a
    s += r1^e1b * r2^e1d * r3^e1a * r4^e1c
    s += r1^e1b * r2^e1d * r3^e1c * r4^e1a

    s += r1^e1c * r2^e1a * r3^e1b * r4^e1d
    s += r1^e1c * r2^e1a * r3^e1d * r4^e1b
    s += r1^e1c * r2^e1b * r3^e1a * r4^e1d
    s += r1^e1c * r2^e1b * r3^e1d * r4^e1a
    s += r1^e1c * r2^e1d * r3^e1a * r4^e1b
    s += r1^e1c * r2^e1d * r3^e1b * r4^e1a

    s += r1^e1d * r2^e1a * r3^e1b * r4^e1c
    s += r1^e1d * r2^e1a * r3^e1c * r4^e1b
    s += r1^e1d * r2^e1b * r3^e1a * r4^e1c
    s += r1^e1d * r2^e1b * r3^e1c * r4^e1a
    s += r1^e1d * r2^e1c * r3^e1a * r4^e1b
    s += r1^e1d * r2^e1c * r3^e1b * r4^e1a

    return ( s )
}

print "**************** First order ****************\n"

print "Symmetric polynomial: <r1> (multiplicity = 4)\n"
ignore = display(sym_poly_3_1(0,1))

print "**************** Second order ****************\n"

print "Symmetric polynomial: <r1^2> (multiplicity = 4)\n"
ignore = display(sym_poly_3_1(0,2))

print "Symmetric polynomial: <r1*r2> (multiplicity = 6)\n"
ignore = display(sym_poly_2_2(1,0))

print "**************** Third order ****************\n"

print "Symmetric polynomial: <r1^3> (multiplicity = 4)\n"
ignore = display(sym_poly_3_1(0,3))

print "Symmetric polynomial: <r1^2*r2> (multiplicity = 12)\n"
ignore = display(sym_poly_2_1_1(0,2,1))

print "Symmetric polynomial: <r1*r2*r3> (multiplicity = 4)\n"
ignore = display(sym_poly_3_1(1,0))

print "**************** Fourth order ****************\n"

print "Symmetric polynomial: <r1^4> (multiplicity = 4)\n"
ignore = display(sym_poly_3_1(0,4))

print "Symmetric polynomial: <r1^3*r2> (multiplicity = 12)\n"
ignore = display(sym_poly_2_1_1(0,3,1))

print "Symmetric polynomial: <r1^2*r2*r3> (multiplicity = 12)\n"
ignore = display(sym_poly_2_1_1(1,2,0))

print "Symmetric polynomial: <r1^2*r2^2> (multiplicity = 6)\n"
ignore = display(sym_poly_2_2(2,0))

print "Symmetric polynomial: <r1*r2*r3*r4> (multiplicity = 1)\n"
ignore = display(sym_poly_4(1))

print "**************** Fifth order ****************\n"

print "Symmetric polynomial: <r1^5> (multiplicity = 4)\n"
ignore = display(sym_poly_3_1(0,5))

print "Symmetric polynomial: <r1^3*r2^2> (multiplicity = 12)\n"
ignore = display(sym_poly_2_1_1(0,3,2))

print "Symmetric polynomial: <r1^3*r2*r3> (multiplicity = 12)\n"
ignore = display(sym_poly_2_1_1(1,3,0))

print "Symmetric polynomial: <r1^2*r2^2*r3> (multiplicity = 12)\n"
ignore = display(sym_poly_2_1_1(2,1,0))

print "**************** Sixth order ****************\n"

print "Symmetric polynomial: <r1^6> (multiplicity = 4)\n"
ignore = display(sym_poly_3_1(0,6))

print "Symmetric polynomial: <r1^2*r2^2*r3^2> (multiplicity = 4)\n"
ignore = display(sym_poly_3_1(2,0))

print "Symmetric polynomial: <r1^3*r2^2*r3> (multiplicity = 24)\n"
ignore = display(sym_poly_1_1_1_1(3,2,1,0))

print "Symmetric polynomial: <r1^3*r2^3> (multiplicity = 6)\n"
ignore = display(sym_poly_2_2(3,0))

print "**************** Seventh order ****************\n"

print "Symmetric polynomial: <r1^3*r2^2*r3^2> (multiplicity = 12)\n"
ignore = display(sym_poly_2_1_1(2,3,0))

print "Symmetric polynomial: <r1^3*r2^3*r3> (multiplicity = 12)\n"
ignore = display(sym_poly_2_1_1(3,1,0))

print "Symmetric polynomial: <r1^2*r2^2*r3^2*r4> (multiplicity = 4)\n"
ignore = display(sym_poly_3_1(2,1))

print "**************** Eighth order ****************\n"

print "Symmetric polynomial: <r1^3*r2^3*r3^2> (multiplicity = 12)\n"
ignore = display(sym_poly_2_1_1(3,2,0))

print "**************** Ninth order ****************\n"

print "Symmetric polynomial: <r1^3*r2^3*r3^3> (multiplicity = 4)\n"
ignore = display(sym_poly_3_1(3,0))

'
} |

bc -q
